@@ -36,17 +36,18 @@ def recommended_column_min_assertion(table, column, profiling_result) -> Recomme
     if column_type == 'numeric':
         total = column_metric['total']
         column_min = column_metric['min']
+        distribution_counts = column_metric['distribution']['counts'] if column_metric['distribution'] else []
 
         count = 0
-        for i, v in enumerate(reversed(column_metric['distribution']['counts'])):
+        for i, v in enumerate(reversed(distribution_counts)):
             count = count + v
-            if i == len(column_metric['distribution']['counts']) // 2:
+            if i == len(distribution_counts) // 2:
                 break
 
         if count / total > 0.95:
             test_function_name = 'assert_column_min_in_range'
             assertion_values = {
-                'min': [round(column_min * 0.9, 4), round(column_min * 1.1, 4)]
+                'min': sorted([round(column_min * 0.9, 4), round(column_min * 1.1, 4)])
             }
             assertion = RecommendedAssertion(test_function_name, assertion_values)
             return assertion
@@ -63,17 +64,18 @@ def recommended_column_max_assertion(table, column, profiling_result) -> Recomme
     if column_type == 'numeric':
         total = column_metric['total']
         column_max = column_metric['max']
+        distribution_counts = column_metric['distribution']['counts'] if column_metric['distribution'] else []
 
         count = 0
-        for i, v in enumerate(column_metric['distribution']['counts']):
+        for i, v in enumerate(distribution_counts):
             count = count + v
-            if i == len(column_metric['distribution']['counts']) // 2:
+            if i == len(distribution_counts) // 2:
                 break
 
         if count / total > 0.95:
             test_function_name = 'assert_column_max_in_range'
             assertion_values = {
-                'max': [round(column_max * 0.9, 4), round(column_max * 1.1, 4)]
+                'max': sorted([round(column_max * 0.9, 4), round(column_max * 1.1, 4)])
             }
             assertion = RecommendedAssertion(test_function_name, assertion_values)
             return assertion
