@@ -7,9 +7,16 @@ import sentry_sdk
 from rich.console import Console
 from rich.syntax import Syntax
 
-from piperider_cli import workspace, __version__, event
+from piperider_cli import __version__, event
+from piperider_cli.adapter import DbtAdapter
+from piperider_cli.assertion_generator import AssertionGenerator
 from piperider_cli.event.track import TrackCommand
 from piperider_cli.guide import Guide
+from piperider_cli.initializer import Initializer
+from piperider_cli.runner import Runner
+from piperider_cli.generate_report import GenerateReport
+from piperider_cli.compare_report import CompareReport
+from piperider_cli.validator import Validator
 
 
 def set_sentry_env():
@@ -65,8 +72,6 @@ def cli(ctx: click.Context):
         click.echo(ctx.get_help())
         Guide().show_tips(ctx.command.name)
 
-    pass
-
 
 cli.command_class = TrackCommand
 
@@ -115,13 +120,13 @@ def init(**kwargs):
         if dbt_project_dir:
             dbt_project_path = os.path.join(dbt_project_dir, "dbt_project.yml")
         else:
-            dbt_project_path = workspace.search_dbt_project_path()
+            dbt_project_path = DbtAdapter.search_dbt_project_path()
 
     if dbt_project_path:
         console.print(f'Use dbt project file: {dbt_project_path}')
     else:
         console.print('[[bold yellow]Skip[/bold yellow]] No dbt project found')
-    config = workspace.init(dbt_project_path=dbt_project_path, dbt_profiles_dir=dbt_profiles_dir)
+    config = Initializer.exec(dbt_project_path=dbt_project_path, dbt_profiles_dir=dbt_profiles_dir)
     if kwargs.get('debug'):
         for ds in config.dataSources:
             console.rule('Configuration')
@@ -144,7 +149,7 @@ def diagnose(**kwargs):
 
     console.print(f'[bold dark_orange]PipeRider Version:[/bold dark_orange] {__version__}')
 
-    if not workspace.debug():
+    if not Validator.diagnose():
         sys.exit(1)
 
 
@@ -169,24 +174,24 @@ def run(**kwargs):
     run_dbt_test = kwargs.get('dbt_test')
     run_dbt_build = kwargs.get('dbt_build')
     dbt_command = 'build' if run_dbt_build else 'test' if run_dbt_test else ''
-    ret = workspace.run(datasource=datasource,
-                        table=table,
-                        output=output,
-                        interaction=not kwargs.get('no_interaction'),
-                        skip_report=skip_report,
-                        skip_recommend=skip_recommend,
-                        dbt_command=dbt_command)
+    ret = Runner.exec(datasource=datasource,
+                      table=table,
+                      output=output,
+                      interaction=not kwargs.get('no_interaction'),
+                      skip_report=skip_report,
+                      skip_recommend=skip_recommend,
+                      dbt_command=dbt_command)
     if not skip_report and ret == 0:
-        workspace.generate_report()
+        GenerateReport.exec()
 
 
 @cli.command(short_help='Generate recommended assertions.')
 @click.option('--input', default=None, type=click.Path(exists=True), help='Specify the raw result file.')
 @add_options(debug_option)
 def generate_assertions(**kwargs):
+    'Generate recommended assertions based on the latest result. By default, the profiling result will be loaded from ".piperider/outputs".'
     input = kwargs.get('input')
-    workspace.generate_recommended_assertions(input=input)
-    pass
+    AssertionGenerator.exec(input=input)
 
 
 @cli.command(short_help='Generate a report.')
@@ -195,8 +200,7 @@ def generate_assertions(**kwargs):
 def generate_report(**kwargs):
     'Generate a report from the latest raw result or specified result. By default, the raw results are saved in ".piperider/outputs".'
 
-    input = kwargs.get('input')
-    workspace.generate_report(input=input)
+    GenerateReport.exec(input=kwargs.get('input'))
 
 
 @cli.command(short_help='Compare two existing reports.')
@@ -208,4 +212,4 @@ def compare_reports(**kwargs):
 
     a = kwargs.get('base')
     b = kwargs.get('input')
-    workspace.compare_report(a=a, b=b)
+    CompareReport.exec(a=a, b=b)
