@@ -1,5 +1,6 @@
 import json
 import os
+import re
 import sys
 from datetime import datetime
 
@@ -7,7 +8,8 @@ import readchar
 from rich.console import Console
 
 import piperider_cli.hack.inquirer as inquirer_hack
-from piperider_cli import datetime_to_str, str_to_datetime
+from piperider_cli import datetime_to_str, str_to_datetime, clone_directory
+from piperider_cli.configuration import PIPERIDER_OUTPUT_PATH, PIPERIDER_COMPARISON_PATH
 
 
 class ProfilerOutput(object):
@@ -190,8 +192,52 @@ class CompareReport(object):
             return answers['profiler_outputs'][0], answers['profiler_outputs'][1]
         return None, None
 
+    def setup_report_variables(self, template_html: str, is_single: bool, data):
+        if isinstance(data, dict):
+            output = json.dumps(data)
+        else:
+            output = data
+        if is_single:
+            variables = f'<script id="piperider-report-variables">\nwindow.PIPERIDER_SINGLE_REPORT_DATA={output};window.PIPERIDER_COMPARISON_REPORT_DATA="";</script>'
+        else:
+            variables = f'<script id="piperider-report-variables">\nwindow.PIPERIDER_SINGLE_REPORT_DATA="";window.PIPERIDER_COMPARISON_REPORT_DATA={output};</script>'
+        html_parts = re.sub(r'<script id="piperider-report-variables">.+?</script>', '#PLACEHOLDER#', template_html).split(
+            '#PLACEHOLDER#')
+        html = html_parts[0] + variables + html_parts[1]
+        return html
+
     def generate_data(self) -> ComparisonData:
         if self.a is None or self.b is None:
             raise Exception("Please select reports to compare first.")
 
         return ComparisonData(self.a.load(), self.b.load())
+
+    @staticmethod
+    def exec(a=None, b=None):
+        console = Console()
+
+        report = CompareReport(PIPERIDER_OUTPUT_PATH, a, b)
+        if not report.select_reports():
+            raise Exception('No valid reports found')
+        comparison_data = report.generate_data()
+
+        from piperider_cli import data
+        report_template_dir = os.path.join(os.path.dirname(data.__file__), 'report', 'comparison-report')
+        with open(os.path.join(report_template_dir, 'index.html')) as f:
+            report_template_html = f.read()
+
+        data_id = comparison_data.id()
+        dir = os.path.join(PIPERIDER_COMPARISON_PATH, data_id)
+        clone_directory(report_template_dir, dir)
+
+        filename = os.path.join(dir, 'index.html')
+        with open(filename, 'w') as f:
+            html = report.setup_report_variables(report_template_html, False, comparison_data.to_json())
+            f.write(html)
+
+        console.print()
+        console.print(f"Comparison report: {filename}")
+
+        # TODO for debugging intermediate data, remove this
+        with open(os.path.join(dir, 'comparison_data.json'), 'w') as f:
+            f.write(comparison_data.to_json())
